
interface ModuleOpts {
    wasmBinary?: ArrayBuffer | Uint8Array | Int8Array;
    mainScriptUrlOrBlob?: string;
    noInitialRun?: boolean;
    fetchSettings?: RequestInit;
    locateFile?: (path: string, scriptDirectory: string) => string;
    onRuntimeInitialized?: () => void;
}
interface PtrType {
    isConst: boolean;
    isReference: boolean;
    isSmartPointer: boolean;
    name: string;
    registeredClass: {
        name: string;
    };
}
interface NativeAPI {
    [key: string]: any;
    $$: {
        count: {
            value: number;
        };
        ptr: number;
        ptrType: PtrType;
        smartPtr?: number;
        smartPtrType?: PtrType;
    };
    clone: () => void;
    delete: () => void;
    deleteLater: () => void;
    isDeleted: () => boolean;
}

declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends (k: infer I) => void ? I : never;

declare const VERSION = "1.11.0";
declare const PCL_VERSION = "1.12.1";

/**
 * https://github.com/emscripten-core/emscripten/blob/main/src/library_fs.js
 * https://nodejs.org/api/fs.html#file-system
 */
declare const writeFile: (path: string, data: string | ArrayBufferView, opts?: {
    flags?: string | undefined;
}) => any;
declare function readFile(path: string, opts?: {
    encoding: 'binary' | 'utf8';
    flags?: string;
}): string | Uint8Array;
declare const unlink: (path: string) => void;
declare const rename: (oldPath: string, newPath: string) => any;
declare const mkdir: (path: string, mode?: number) => void;
declare const rmdir: (path: string) => void;
declare const readdir: (path: string) => any;
declare function stat(path: string, dontFollow?: boolean): {
    isDir: boolean;
    isFile: boolean;
    blksize: number;
    blocks: number;
    dev: number;
    gid: number;
    ino: number;
    mode: 33206;
    nlink: number;
    rdev: number;
    size: number;
    uid: number;
    atime: Date;
    mtime: Date;
    ctime: Date;
};

declare const index_d_writeFile: typeof writeFile;
declare const index_d_readFile: typeof readFile;
declare const index_d_unlink: typeof unlink;
declare const index_d_rename: typeof rename;
declare const index_d_mkdir: typeof mkdir;
declare const index_d_rmdir: typeof rmdir;
declare const index_d_readdir: typeof readdir;
declare const index_d_stat: typeof stat;
declare namespace index_d {
  export {
    index_d_writeFile as writeFile,
    index_d_readFile as readFile,
    index_d_unlink as unlink,
    index_d_rename as rename,
    index_d_mkdir as mkdir,
    index_d_rmdir as rmdir,
    index_d_readdir as readdir,
    index_d_stat as stat,
  };
}

declare class PointXY {
    x: number;
    y: number;
    private readonly _PointXY;
    constructor(x?: number, y?: number);
}
declare class PointXYZ {
    x: number;
    y: number;
    z: number;
    private readonly _PointXYZ;
    constructor(x?: number, y?: number, z?: number);
}
declare class PointXYZI {
    x: number;
    y: number;
    z: number;
    intensity: number;
    private readonly _PointXYZI;
    constructor(x?: number, y?: number, z?: number, intensity?: number);
}
declare class PointXYZL {
    x: number;
    y: number;
    z: number;
    label: number;
    private readonly _PointXYZL;
    constructor(x?: number, y?: number, z?: number, label?: number);
}
declare class InterestPoint {
    x: number;
    y: number;
    z: number;
    strength: number;
    private readonly _InterestPoint;
    constructor(x?: number, y?: number, z?: number, strength?: number);
}
declare class PointXYZRGB {
    x: number;
    y: number;
    z: number;
    r: number;
    g: number;
    b: number;
    private readonly _PointXYZRGB;
    constructor(x?: number, y?: number, z?: number, r?: number, g?: number, b?: number);
}
declare class PointXYZRGBA {
    x: number;
    y: number;
    z: number;
    r: number;
    g: number;
    b: number;
    a: number;
    private readonly _PointXYZRGBA;
    constructor(x?: number, y?: number, z?: number, r?: number, g?: number, b?: number, a?: number);
}
declare class PointXYZRGBL {
    x: number;
    y: number;
    z: number;
    r: number;
    g: number;
    b: number;
    label: number;
    private readonly _PointXYZRGBL;
    constructor(x?: number, y?: number, z?: number, r?: number, g?: number, b?: number, label?: number);
}
declare class Normal {
    normalX: number;
    normalY: number;
    normalZ: number;
    curvature: number;
    private readonly _Normal;
    constructor(normalX?: number, normalY?: number, normalZ?: number, curvature?: number);
}
declare class PointNormal {
    x: number;
    y: number;
    z: number;
    normalX: number;
    normalY: number;
    normalZ: number;
    curvature: number;
    private readonly _PointNormal;
    constructor(x?: number, y?: number, z?: number, normalX?: number, normalY?: number, normalZ?: number, curvature?: number);
}
declare class PointXYZLNormal {
    x: number;
    y: number;
    z: number;
    label: number;
    normalX: number;
    normalY: number;
    normalZ: number;
    curvature: number;
    private readonly _PointXYZLNormal;
    constructor(x?: number, y?: number, z?: number, label?: number, normalX?: number, normalY?: number, normalZ?: number, curvature?: number);
}
declare class PointXYZINormal {
    x: number;
    y: number;
    z: number;
    intensity: number;
    normalX: number;
    normalY: number;
    normalZ: number;
    curvature: number;
    private readonly _PointXYZINormal;
    constructor(x?: number, y?: number, z?: number, intensity?: number, normalX?: number, normalY?: number, normalZ?: number, curvature?: number);
}
declare class PointXYZRGBNormal {
    x: number;
    y: number;
    z: number;
    r: number;
    g: number;
    b: number;
    normalX: number;
    normalY: number;
    normalZ: number;
    curvature: number;
    private readonly _PointXYZRGBNormal;
    constructor(x?: number, y?: number, z?: number, r?: number, g?: number, b?: number, normalX?: number, normalY?: number, normalZ?: number, curvature?: number);
}
declare class PointSurfel {
    x: number;
    y: number;
    z: number;
    normalX: number;
    normalY: number;
    normalZ: number;
    r: number;
    g: number;
    b: number;
    a: number;
    radius: number;
    confidence: number;
    curvature: number;
    private readonly _PointSurfel;
    constructor(x?: number, y?: number, z?: number, normalX?: number, normalY?: number, normalZ?: number, r?: number, g?: number, b?: number, a?: number, radius?: number, confidence?: number, curvature?: number);
}
declare type PointTypes = PointXY | PointXYZ | PointXYZI | InterestPoint | PointXYZL | PointXYZRGB | PointXYZRGBA | PointXYZRGBL | Normal | PointNormal | PointXYZRGBNormal | PointXYZINormal | PointXYZLNormal | PointSurfel;
declare type PointTypesTypeof = typeof PointXY | typeof PointXYZ | typeof PointXYZI | typeof InterestPoint | typeof PointXYZL | typeof PointXYZRGB | typeof PointXYZRGBA | typeof PointXYZRGBL | typeof Normal | typeof PointNormal | typeof PointXYZRGBNormal | typeof PointXYZINormal | typeof PointXYZLNormal | typeof PointSurfel;
declare type XYZPointTypes = PointXYZ | PointXYZI | PointXYZL | PointXYZRGBA | PointXYZRGB | PointXYZRGBL | InterestPoint | PointNormal | PointXYZRGBNormal | PointXYZINormal | PointXYZLNormal | PointSurfel;
declare type XYZPointTypesTypeof = typeof PointXYZ | typeof PointXYZI | typeof PointXYZL | typeof PointXYZRGBA | typeof PointXYZRGB | typeof PointXYZRGBL | typeof InterestPoint | typeof PointNormal | typeof PointXYZRGBNormal | typeof PointXYZINormal | typeof PointXYZLNormal | typeof PointSurfel;
declare type XYZLPointTypes = PointXYZL | PointXYZRGBL | PointXYZLNormal;
declare type RGBPointTypes = PointXYZRGB | PointXYZRGBA | PointXYZRGBL | PointXYZRGBNormal | PointSurfel;
declare type NormalPointTypes = Normal | PointNormal | PointXYZRGBNormal | PointXYZINormal | PointXYZLNormal | PointSurfel;
declare const pointTypeMap: {
    PointXY: typeof PointXY;
    PointXYZ: typeof PointXYZ;
    PointXYZI: typeof PointXYZI;
    InterestPoint: typeof InterestPoint;
    PointXYZL: typeof PointXYZL;
    PointXYZRGB: typeof PointXYZRGB;
    PointXYZRGBA: typeof PointXYZRGBA;
    PointXYZRGBL: typeof PointXYZRGBL;
    Normal: typeof Normal;
    PointNormal: typeof PointNormal;
    PointXYZRGBNormal: typeof PointXYZRGBNormal;
    PointXYZINormal: typeof PointXYZINormal;
    PointXYZLNormal: typeof PointXYZLNormal;
    PointSurfel: typeof PointSurfel;
};

declare class Manager {
    private _native;
    constructor(_native: NativeAPI);
    /**
     * Delete a C ++ object
     * {@link https://emscripten.org/docs/porting/connecting_cpp_and_javascript/embind.html?highlight=clone#memory-management}
     */
    delete(): void;
    isDeleted(): boolean;
    /**
     * {@link https://emscripten.org/docs/porting/connecting_cpp_and_javascript/embind.html?highlight=clone#cloning-and-reference-counting}
     * @returns A new handle
     */
    clone(): void;
}

declare class Vector<T> {
    _native: NativeAPI;
    manager: Manager;
    constructor(_native: NativeAPI);
    get size(): number;
    set(index: number, value: T): boolean;
    get(index: number): T;
    push(value: T): void;
    isEmpty(): boolean;
    resize(count: number, value?: T): void;
    clear(): void;
}
declare class Indices extends Vector<number> {
    constructor(native?: NativeAPI);
}
declare class PCLHeader {
    _native: NativeAPI;
    manager: Manager;
    constructor(_native: NativeAPI);
    get seq(): number;
    get stamp(): bigint;
    get frameId(): string;
}
declare class PointIndices {
    _native: NativeAPI;
    manager: Manager;
    header: PCLHeader;
    indices: Indices;
    constructor(native?: NativeAPI);
}
declare class Points<T extends PointTypes> extends Vector<T> {
    private readonly _PT;
    constructor(_native: NativeAPI);
    get(index: number): T;
}
declare class PointCloud<T extends PointTypes = PointXYZ> {
    readonly _PT: PointTypesTypeof;
    _native: NativeAPI;
    manager: Manager;
    private _points?;
    constructor(_PT?: PointTypesTypeof, _native?: NativeAPI);
    get isOrganized(): boolean;
    get isDense(): boolean;
    set width(v: number);
    get width(): number;
    set height(v: number);
    get height(): number;
    get header(): PCLHeader;
    get size(): number;
    get points(): Points<T>;
    /**
     * Removes all points in a cloud and sets the width and height to 0.
     */
    clear(): void;
    /**
     * Resizes the container to contain `count` elements
     * @params count - New size of the point cloud
     * @params pt - The value to initialize the new points with
     */
    resize(count: number, pt?: T): void;
    /**
     * Insert a new point in the cloud, at the end of the container.
     * @description This breaks the organized structure of the cloud by setting the height to 1!
     * @params pt - The point to insert
     */
    addPoint(pt?: T | null): void;
}

declare class ModelCoefficients {
    _native: NativeAPI;
    constructor();
    get header(): PCLHeader;
    get values(): Vector<unknown>;
}

declare function computeCloudResolution(cloud: PointCloud<XYZPointTypes>): number;
declare function toXYZPointCloud(cloudIn: PointCloud<XYZPointTypes>, cloudOut: PointCloud<PointXYZ>): void;

interface PCDHeader {
    version: number;
    fields: string[];
    type: ('F' | 'I' | 'U')[];
    count: number[];
    size: number[];
    offset: number[];
    width: number;
    height: number;
    points: number;
    viewpoint: string;
    data: 'ascii' | 'binary' | 'binary_compressed';
}
/**
 * {@link https://github.com/mrdoob/three.js/blob/master/examples/jsm/loaders/PCDLoader.js}
 * {@link https://pointclouds.org/documentation/tutorials/pcd_file_format.html}
 *
 * @param buffer pcd file data
 * @returns pcd header
 */
declare function readPCDHeader(buffer: ArrayBuffer): PCDHeader | undefined;

declare function loadPCDFile<T extends PointTypes = PointXYZ & Partial<UnionToIntersection<PointTypes>>>(filename: string, _PT?: PointTypesTypeof): PointCloud<T>;
declare function savePCDFile(filename: string, cloud: PointCloud<PointTypes>, binaryMode?: boolean): boolean;
declare function savePCDFileASCII(filename: string, cloud: PointCloud<PointTypes>): boolean;
declare function savePCDFileBinary(filename: string, cloud: PointCloud<PointTypes>): boolean;
declare function savePCDFileBinaryCompressed(filename: string, cloud: PointCloud<PointTypes>): boolean;
declare function readPCDFileHeader(filename: string): PCDHeader | undefined;
declare function loadPCDData<T extends PointTypes = PointXYZ & Partial<UnionToIntersection<PointTypes>>>(data: ArrayBuffer, _PT?: PointTypesTypeof): PointCloud<T>;
declare function savePCDData(cloud: PointCloud<PointTypes>, binaryMode?: boolean): string | Uint8Array;
declare function savePCDDataASCII(cloud: PointCloud<PointTypes>): string | Uint8Array;
declare function savePCDDataBinary(cloud: PointCloud<PointTypes>): string | Uint8Array;
declare function savePCDDataBinaryCompressed(cloud: PointCloud<PointTypes>): string | Uint8Array;

declare enum SacMethodTypes {
    SAC_RANSAC = 0,
    SAC_LMEDS = 1,
    SAC_MSAC = 2,
    SAC_RRANSAC = 3,
    SAC_RMSAC = 4,
    SAC_MLESAC = 5,
    SAC_PROSAC = 6
}
declare enum SacModelTypes {
    SACMODEL_PLANE = 0,
    SACMODEL_LINE = 1,
    SACMODEL_CIRCLE2D = 2,
    SACMODEL_CIRCLE3D = 3,
    SACMODEL_SPHERE = 4,
    SACMODEL_CYLINDER = 5,
    SACMODEL_CONE = 6,
    SACMODEL_TORUS = 7,
    SACMODEL_PARALLEL_LINE = 8,
    SACMODEL_PERPENDICULAR_PLANE = 9,
    SACMODEL_PARALLEL_LINES = 10,
    SACMODEL_NORMAL_PLANE = 11,
    SACMODEL_NORMAL_SPHERE = 12,
    SACMODEL_REGISTRATION = 13,
    SACMODEL_REGISTRATION_2D = 14,
    SACMODEL_PARALLEL_PLANE = 15,
    SACMODEL_NORMAL_PARALLEL_PLANE = 16,
    SACMODEL_STICK = 17
}

declare class KdTree$1<T extends XYZPointTypes> {
    _native: NativeAPI;
    manager: Manager;
    constructor(_native: NativeAPI);
    setInputCloud(cloud: PointCloud<T>): void;
    getInputCloud(): PointCloud<T>;
}

declare class KdTreeFLANN<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends KdTree$1<T> {
    constructor(_PT?: XYZPointTypesTypeof, sorted?: boolean);
    setSortedResults(sorted: boolean): void;
    setEpsilon(eps: number): void;
    nearestKSearch(point: T, k: number): {
        indices: Indices;
        distances: Vector<number>;
    };
    radiusSearch(point: T, radius: number, maxNN?: number): {
        indices: Indices;
        distances: Vector<number>;
    };
}

declare class Search<T extends XYZPointTypes> {
    _native: NativeAPI;
    manager: Manager;
    constructor(_native: NativeAPI);
    getInputCloud(): PointCloud<T>;
    getName(): void;
    getSortedResults(): void;
}

declare class KdTree<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends Search<T> {
    constructor(_PT?: XYZPointTypesTypeof, sorted?: boolean);
    setSortedResults(sorted: boolean): void;
    setEpsilon(eps: number): void;
    setInputCloud(cloud: PointCloud<T>): void;
    nearestKSearch(point: T, k: number): {
        indices: Indices;
        distances: Vector<number>;
    };
    radiusSearch(point: T, radius: number, maxNN?: number): {
        indices: Indices;
        distances: Vector<number>;
    };
}

declare class PCLBase<T extends PointTypes> {
    _native: NativeAPI;
    manager: Manager;
    constructor(_native: NativeAPI);
    setInputCloud(cloud: PointCloud<T>): void;
    getInputCloud(): PointCloud<T>;
}

declare abstract class FilterBase<T extends PointTypes> extends PCLBase<T> {
    protected abstract _PT?: PointTypesTypeof;
    filter(cloud?: PointCloud<T>): PointCloud<T> | null;
    getRemovedIndices(): Indices;
}

declare class FilterIndices<T extends PointTypes> extends FilterBase<T> {
    protected _PT?: PointTypesTypeof;
    setNegative(negative?: boolean): void;
    getNegative(): boolean;
    setKeepOrganized(keepOrganized?: boolean): void;
    getKeepOrganized(): boolean;
    setUserFilterValue(value: number): void;
}

declare class PassThrough<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setFilterFieldName(fieldName: string): any;
    getFilterFieldName(): string | null;
    setFilterLimits(min: number, max: number): any;
    getFilterLimits(): [number, number];
}

declare class VoxelGrid<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterBase<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof);
    setLeafSize(lx: number, ly: number, lz: number): any;
    setDownsampleAllData(downsample: boolean): any;
    getDownsampleAllData(): boolean;
    setMinimumPointsNumberPerVoxel(minPointsPerVoxel: number): any;
    getMinimumPointsNumberPerVoxel(): number;
}

declare class StatisticalOutlierRemoval<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setMeanK(nrK: number): any;
    getMeanK(): number;
    setStddevMulThresh(stddevMult: number): any;
    getStddevMulThresh(): number;
}

declare class RadiusOutlierRemoval<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setRadiusSearch(radius: number): any;
    getRadiusSearch(): number;
    setMinNeighborsInRadius(minPts?: number): any;
    getMinNeighborsInRadius(): number;
}

declare class UniformSampling<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterBase<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setRadiusSearch(radius: number): void;
}

declare class RandomSample<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setSample(sample: number): any;
    getSample(): number;
    setSeed(seed: number): any;
    getSeed(): number;
}

declare class GridMinimum<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, resolution?: number);
    setResolution(resolution: number): any;
    getResolution(): number;
}

declare class LocalMaximum<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterIndices<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof, extractRemovedIndices?: boolean);
    setRadius(radius: number): any;
    getRadius(): number;
}

declare class ApproximateVoxelGrid<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends FilterBase<T> {
    protected _PT: XYZPointTypesTypeof;
    constructor(_PT?: XYZPointTypesTypeof);
    setLeafSize(lx: number, ly: number, lz: number): any;
    setDownsampleAllData(downsample: boolean): any;
    getDownsampleAllData(): boolean;
}

declare function removeNaNFromPointCloud<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>>(cloudIn: PointCloud<T>, cloudOut?: PointCloud<T>, indices?: Indices): {
    cloud: PointCloud<T>;
    indices: Indices;
};
declare function removeNaNNormalsFromPointCloud<T extends NormalPointTypes = PointNormal & Partial<UnionToIntersection<NormalPointTypes>>>(cloudIn: PointCloud<T>, cloudOut?: PointCloud<T>, indices?: Indices): {
    cloud: PointCloud<T>;
    indices: Indices;
};

declare class Keypoint<T extends XYZPointTypes> extends PCLBase<T> {
    setSearchMethod(tree: KdTree$1<T>): void;
    getSearchMethod(): KdTree$1<T>;
    getSearchParameter(): number;
    getKeypointsIndices(): PointIndices;
    setKSearch(k: number): void;
    getKSearch(): number;
    setRadiusSearch(radius: number): void;
    getRadiusSearch(): number;
    compute(cloud: PointCloud<T>): void;
}

declare class ISSKeypoint3D<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends Keypoint<T> {
    constructor(_PT?: XYZPointTypesTypeof, salientRadius?: number);
    setSalientRadius(salientRadius: number): void;
    setNonMaxRadius(radius: number): void;
    setNormalRadius(radius: number): void;
    setBorderRadius(radius: number): void;
    setMinNeighbors(minNeighbors: number): void;
    setThreshold21(gamma21: number): void;
    setThreshold32(gamma32: number): void;
    setAngleThreshold(angle: number): void;
}

declare class MinCutSegmentation<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends PCLBase<T> {
    _native: NativeAPI;
    constructor(PT?: XYZPointTypesTypeof);
    setSigma(sigma: number): void;
    getSigma(): number;
    setRadius(radius: number): void;
    getRadius(): number;
    setSourceWeight(weight: number): void;
    getSourceWeight(): number;
    setSearchMethod(tree: string | null): void;
    getSearchMethod(): string | null;
    setNumberOfNeighbours(neighbourNumber: number): void;
    getNumberOfNeighbours(): number;
    setForegroundPoints(cloud: PointCloud<T>): void;
    getForegroundPoints(): Points<T>;
    setBackgroundPoints(cloud: PointCloud<T>): void;
    getBackgroundPoints(): Points<T>;
    extract(): Vector<unknown>;
    getMaxFlow(): number;
    getColoredCloud(): PointCloud<PointXYZRGB>;
}

declare class SACSegmentation<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> extends PCLBase<T> {
    private _modelType;
    private _methodType;
    private _optimizeCoefficients;
    private _epsAngle;
    private _threshold;
    private _maxIterations;
    private _probability;
    private _samplesRadius;
    private _samplesRadiusSearch?;
    private _minRadius;
    private _maxRadius;
    constructor(random?: boolean);
    setModelType(model: SacModelTypes): void;
    getModelType(): SacModelTypes;
    setMethodType(method: SacMethodTypes): void;
    getMethodType(): SacMethodTypes;
    setDistanceThreshold(threshold: number): void;
    getDistanceThreshold(): number;
    setMaxIterations(maxIterations: number): void;
    getMaxIterations(): number;
    setProbability(probability: number): void;
    getProbability(): number;
    setRadiusLimits(minRadius: number, maxRadius: number): void;
    getRadiusLimits(): {
        minRadius: number;
        maxRadius: number;
    };
    setSamplesMaxDist(radius: number, search: Search<T>): void;
    getSamplesMaxDist(): {
        radius: number;
        search: Search<T> | undefined;
    };
    setEpsAngle(epsAngle: number): void;
    getEpsAngle(): number;
    setOptimizeCoefficients(optimizeCoefficients: boolean): void;
    getOptimizeCoefficients(): boolean;
    segment(inliers: PointIndices, modelCoefficients: ModelCoefficients): void;
    /**
     * Only supports `PointXYZ` type,
     * if it is not `PointXYZ` type, it will use `toXYZPointCloud` method to convert to `PointXYZ`
     */
    setInputCloud(cloud: PointCloud<T>): void;
}

declare class IterativeClosestPoint<T extends XYZPointTypes = PointXYZ & Partial<UnionToIntersection<XYZPointTypes>>> {
    _native: NativeAPI;
    constructor(_PT?: XYZPointTypesTypeof);
    setInputSource(cloud: PointCloud<T>): void;
    setInputTarget(cloud: PointCloud<T>): void;
    getFinalTransformation(): string | null;
    getFitnessScore(): number;
    hasConverged(): boolean;
    setUseReciprocalCorrespondences(useReciprocalCorrespondence: boolean): void;
    getUseReciprocalCorrespondences(): boolean;
    align(output?: PointCloud<T>): PointCloud<T>;
}

declare let isInitialized: boolean;
interface InitOptions {
    /**
     * ArrayBuffer for pcl-core.wasm file.
     */
    arrayBuffer?: ModuleOpts['wasmBinary'];
    /**
     * URL for pcl-core.wasm file.
     */
    url?: string;
    /**
     * Override the default settings object used when fetching the Wasm module from the network.
     *
     * @default { credentials: 'same-origin' }
     */
    fetchOptions?: ModuleOpts['fetchSettings'];
    onsuccess?: () => void;
    onerror?: (error: unknown) => void;
}
declare function init(options?: InitOptions): Promise<void>;
declare function destroy(): void;

export { ApproximateVoxelGrid, GridMinimum, ISSKeypoint3D, Indices, InitOptions, InterestPoint, IterativeClosestPoint, KdTreeFLANN, LocalMaximum, MinCutSegmentation, ModelCoefficients, Normal, NormalPointTypes, PCDHeader, PCLHeader, PCL_VERSION, PassThrough, PointCloud, PointIndices, PointNormal, PointSurfel, PointTypes, PointTypesTypeof, PointXY, PointXYZ, PointXYZI, PointXYZINormal, PointXYZL, PointXYZLNormal, PointXYZRGB, PointXYZRGBA, PointXYZRGBL, PointXYZRGBNormal, Points, RGBPointTypes, RadiusOutlierRemoval, RandomSample, SACSegmentation, SacMethodTypes, SacModelTypes, KdTree as SearchKdTree, StatisticalOutlierRemoval, UniformSampling, VERSION, VoxelGrid, XYZLPointTypes, XYZPointTypes, XYZPointTypesTypeof, computeCloudResolution, destroy, index_d as fs, init, isInitialized, loadPCDData, loadPCDFile, pointTypeMap, readPCDFileHeader, readPCDHeader, removeNaNFromPointCloud, removeNaNNormalsFromPointCloud, savePCDData, savePCDDataASCII, savePCDDataBinary, savePCDDataBinaryCompressed, savePCDFile, savePCDFileASCII, savePCDFileBinary, savePCDFileBinaryCompressed, toXYZPointCloud };
  
export as namespace PCL;
